/**
* CGI.java
* 1999/11/20 by Nobuyuki Tsuchimura (tutimura@logopt.com)
*/

import java.net.*;
import java.io.*;
import java.util.*;


public class CGI {
    URL url;

    public CGI( URL url ) {
	this.url = url;
    }

    public CGI( String spec ) {
	this( null, spec );
    }

    public CGI( URL context, String spec ) {
	try {
	    url = new URL( context, spec );
	} catch( MalformedURLException e ) {
	    error( "MalformedURLException" );
	}
    }


    private void error( String message ) {
	System.err.println( "CGI:" + message );
    }


    private URLConnection openConnection( String[] post ) throws IOException {
	URLConnection connect = url.openConnection();
	connect.setDoOutput( true );
	PrintStream out = new PrintStream( connect.getOutputStream() );
	for ( int i=0; post!=null && i<post.length; i++ ) {
	    if ( post[i] != null ) out.println( post[i] );
	}
	out.close();
	return connect;
    }

    private Vector inputVector( URLConnection connect ) throws IOException {
	Vector vector = new Vector();
	DataInputStream in = new DataInputStream( connect.getInputStream() );
	String s;
	while ( (s=in.readLine()) != null ) vector.addElement( s );
	in.close();
	return vector;
    }

    public String[] postGetStrings( String[] str ) {
	Vector inVector;
	try {
	    URLConnection connect = openConnection( str );
	    inVector = inputVector( connect );
	} catch( IOException e ) {
	    error( "IOException" );
	    return null;
	}
	String[] inString = new String[ inVector.size() ];
	inVector.copyInto( inString );
	inVector = null;
	return inString;
    }

  
    protected DataInputStream inputStream;

    public synchronized boolean post( String[] str ) {
//	if ( inputStream != null ) return false;
	try {
	    URLConnection connection = openConnection( str );
	    inputStream = new DataInputStream( connection.getInputStream() );
	} catch( IOException e ) {
	    error( "IOException" );
	    return false;
	}
	return true;
    }

    protected String unreadBuff;

    public synchronized String readLine() {
	String s = unreadBuff;
	if ( s != null ) { unreadBuff = null; return s; }
	try {
	    if ( (s=inputStream.readLine()) != null ) return s; // OK
	    inputStream.close();
	} catch( IOException e ) {
	    error( "IOException" );
	}
	inputStream = null;
	return null;
    }

    public synchronized boolean unreadLine( String str ) {
	if ( unreadBuff != null ) return false;
	unreadBuff = str;
	return true;
    }
}
